// bayesian factor analysis for "Measuring University Quality"
// multilevel model for stacked rater data
// unconstrained model 
// with correlated intercepts and slopes
// stan ref manual 5.11
// no intercepts for country
// with fitted and predicted values
// with J error terms

data{
  int<lower=1> N;               // number of universities
  int<lower=1> J;               // number of raters
  int<lower=1> K;  	        	// number of ratings
  int<lower=1,upper=N> ii[K];   // university for rating k
  int<lower=1,upper=J> jj[K];   // rater for rating k
  real x[K];                    // ratings
  vector<lower=1>[N] eta_1;     // vector of 1s
  vector[2] mu_Beta;
}

parameters{
  vector[N] eta;                // factor scores
  vector<lower=0>[J] sigma_x;   // sd of residuals, 
  corr_matrix[2] Omega_Beta;    // correlation matrix for Beta pars
  vector<lower=0>[2] tau_Beta;  // cor -> cov conversion Beta
  vector[2] Beta[J];            // rater intercepts and slopes
}

transformed parameters{
  real SD_eta;                  // sd of factors
  matrix[N,2] Eta;
  matrix[2,2] Sigma_Beta;
  real x_hat[K];                // predicted values
 
  SD_eta <- sd(eta);
  Eta <- append_col(eta_1, eta);
  Sigma_Beta <- quad_form_diag(Omega_Beta, tau_Beta);
  for (k in 1:K)  
    x_hat[k] <- Eta[ii[k]] * Beta[jj[k]];
}

model{
  sigma_x ~ uniform(0, 100);
  eta ~ normal(0, 1);           // identify scale and location  

  tau_Beta ~ cauchy(0, 2.5);
  Omega_Beta ~ lkj_corr(2);
  for (j in 1:J)
    Beta[j] ~ multi_normal(mu_Beta, Sigma_Beta);

  for (k in 1:K)     
    x[k] ~ normal(x_hat[k], sigma_x[jj[k]]);
}

generated quantities {
  vector[K] x_pred;
  
  for (k in 1:K)
    x_pred[k] <- normal_rng(Eta[ii[k]] * Beta[jj[k]], sigma_x[jj[k]]);
}
